@@ -67,6 +67,7 @@ class ADCSettings(util.DisableNewAttr):
     def __str__(self):
         return self.__repr__()
 
+
     @property
     def samples(self):
         """Number of samples to store."""
@@ -587,6 +588,16 @@ class CWNano(ScopeTemplate, util.DisableNewAttr):
     def _getNAEUSB(self):
         return self._cwusb
 
+    @property
+    def latest_fw(self):
+        from chipwhisperer.hardware.firmware.cwnano import fwver
+        return {"major": fwver[0], "minor": fwver[1]}
+
+    @property
+    def fw_version(self):
+        a = self._cwusb.readFwVersion()
+        return {"major": a[0], "minor": a[1], "debug": a[2]}
+
     def _con(self, sn=None):
         try:
             possible_sn = self._cwusb.get_possible_devices(idProduct=[0xACE0])
@@ -634,6 +645,16 @@ class CWNano(ScopeTemplate, util.DisableNewAttr):
 
         self._lasttrace = self._cwusb.cmdReadMem(0, self.adc.samples)
 
+        # can just keep rerunning this until it works I think
+        i = 0
+        while len(self._lasttrace) < self.adc.samples:
+            logging.debug("couldn't read ADC data from Nano, retrying...")
+
+            self._lasttrace = self._cwusb.cmdReadMem(0, self.adc.samples)
+            i+= 1
+            if i > 20:
+                logging.warning("Couldn't read trace data back from Nano")
+                return True
         self._lasttrace = np.array(self._lasttrace) / 256.0 - 0.5
 
         #self.newDataReceived(0, self._lasttrace, 0, self.adc.clk_freq)
@@ -651,6 +672,7 @@ class CWNano(ScopeTemplate, util.DisableNewAttr):
 
     def _dict_repr(self):
         dict = OrderedDict()
+        dict['fw_version'] = self.fw_version
         dict['io']    = self.io._dict_repr()
         dict['adc']   = self.adc._dict_repr()
         dict['glitch'] = self.glitch._dict_repr()
