@@ -59,10 +59,16 @@ class SimpleSerial(TargetTemplate, util.DisableNewAttr):
       * :meth:`target.simpleserial_wait_ack <.SimpleSerial.simpleserial_wait_ack>`
       * :meth:`target.simpleserial_write <.SimpleSerial.simpleserial_write>`
       * :meth:`target.simpleserial_read <.SimpleSerial.simpleserial_read>`
+      * :meth:`target.simpleserial_read_witherrors <.SimpleSerial.simpleserial_read_witherrors>`
       * :meth:`target.set_key <.SimpleSerial.set_key>`
       * :meth:`target.close <.SimpleSerial.close>`
       * :meth:`target.con <.SimpleSerial.con>`
 
+    .. warning::
+        The CWLite, CW1200, and CWNano have a 128 byte read buffer and a 128 
+        byte send buffer. If the read buffer overflows, a warning message
+        will be printed. Prior to firmware 0.20, the send buffer can silently
+        overflow.
     """
     _name = "Simple Serial"
 
@@ -265,7 +271,13 @@ class SimpleSerial(TargetTemplate, util.DisableNewAttr):
         if data[0] != 'z':
             logging.error("Ack error: {}".format(data))
             return None
-        return int(data[1:3], 16)
+        ret = None
+        try:
+            ret = int(data[1:3], 16)
+        except ValueError:
+            logging.error("Ack error, couldn't decode return {}".format(data))
+            return None
+        return ret
 
     def simpleserial_write(self, cmd, num, end='\n'):
         """ Writes a simpleserial command to the target over serial.
@@ -445,17 +457,19 @@ class SimpleSerial(TargetTemplate, util.DisableNewAttr):
                     payload[i] = int(response[idx:(idx + 2)], 16)
                 except ValueError as e:
                     payload = None
-                    logging.warning("ValueError: {}".format(e))
+                    valid = False
+                    break
+                    #logging.warning("ValueError: {}".format(e))
 
                 idx += 2
 
-            if len(end) > 0:
+            if valid and (len(end) > 0):
                 if response[(idx):(idx + len(end))] != end:
                     logging.warning("Unexpected end to command: {}".format(
                         response[(idx):(idx + len(end))]))
                     payload = None
 
-            if ack:
+            if ack and valid:
                 rv = self.simpleserial_wait_ack(timeout)
 
         # return payload, valid, response
