@@ -39,6 +39,7 @@ import usb.util
 from chipwhisperer.hardware.firmware import cwlite as fw_cwlite
 from chipwhisperer.hardware.firmware import cw1200 as fw_cw1200
 from chipwhisperer.hardware.firmware import cw305  as fw_cw305
+from chipwhisperer.hardware.firmware import cwnano  as fw_nano
 
 
 
@@ -71,6 +72,7 @@ NEWAE_PIDS = {
     0xACE2: {'name': "ChipWhisperer-Lite",     'fwver': fw_cwlite.fwver},
     0xACE3: {'name': "ChipWhisperer-CW1200",   'fwver': fw_cw1200.fwver},
     0xC305: {'name': "CW305 Artix FPGA Board", 'fwver': fw_cw305.fwver},
+    0xACE0: {'name': "ChipWhisperer-Nano", 'fwver': fw_nano.fwver},
 }
 
 class NAEUSB_Serializer_base(object):
@@ -90,6 +92,7 @@ class NAEUSB_Serializer_base(object):
     GET_POSSIBLE_DEVICES = 0xF6
     WRITE_BULK = 0xF7
     READ_BULK = 0xF8
+    CMD_WRITE_MEM_SAM3U = 0xF9
 
 
     ACK = 0xA0
@@ -210,6 +213,19 @@ class NAEUSB_Serializer(NAEUSB_Serializer_base):
         cmdpacket = self.make_cmd(self.CMD_WRITE_MEM, payload)
         return self.process_rx(self.txrx(tx=cmdpacket))
 
+    def cmdWriteSam3U(self, addr, data):
+        """
+        Send command to write memory over the SAM3U. 
+        """
+
+        dlen = len(data)
+
+        payload = [addr]
+        payload.extend(data)
+        cmdpacket = self.make_cmd(self.CMD_WRITE_MEM_SAM3U, payload)
+        return self.process_rx(self.txrx(tx=cmdpacket))
+
+
     def writeBulk(self, data):
         """
         Low-level function.
@@ -241,6 +257,7 @@ class NAEUSB_Backend(NAEUSB_Serializer_base):
     CMD_READMEM_CTRL = 0x12
     CMD_WRITEMEM_CTRL = 0x13
     CMD_MEMSTREAM = 0x14
+    CMD_WRITEMEM_CTRL_SAM3U = 0x15
 
     def __init__(self):
         self._usbdev = None
@@ -289,6 +306,10 @@ class NAEUSB_Backend(NAEUSB_Serializer_base):
                 addr = payload[0]
                 data = payload[1:]
                 self.cmdWriteMem(addr, data)
+            elif cmd == self.CMD_WRITE_MEM_SAM3U:
+                addr = payload[0]
+                data = payload[1:]
+                self.cmdWriteSam3U(addr, data)
             elif cmd == self.GET_POSSIBLE_DEVICES:
                 response = self.get_possible_devices(payload)
             elif cmd == self.OPEN:
@@ -505,6 +526,29 @@ class NAEUSB_Backend(NAEUSB_Serializer_base):
 
         return data
 
+    def cmdWriteSam3U(self, addr, data):
+        """
+        Send command to write memory over memory of SAMU3. 
+        """
+
+        dlen = len(data)
+
+        if dlen < 48:
+            cmd = self.CMD_WRITEMEM_CTRL_SAM3U
+        else:
+            cmd = self.ERROR
+
+        # ADDR/LEN written LSB first
+        pload = packuint32(dlen)
+        pload.extend(packuint32(addr))
+
+        if cmd == self.CMD_WRITEMEM_CTRL_SAM3U:
+            pload.extend(data)
+
+        self.sendCtrl(cmd, data=pload)
+        
+        return data
+
 
     def cmdWriteBulk(self, data):
         """
@@ -540,6 +584,7 @@ class NAEUSB(object):
     CMD_READMEM_CTRL = 0x12
     CMD_WRITEMEM_CTRL = 0x13
     CMD_MEMSTREAM = 0x14
+    CMD_WRITEMEM_CTRL_SAM3U = 0x15
 
     stream = False
 
@@ -606,7 +651,8 @@ class NAEUSB(object):
         latest = fwver[0] > fw_latest[0] or (fwver[0] == fw_latest[0] and fwver[1] >= fw_latest[1])
         if not latest:
             logging.warning('Your firmware is outdated - latest is %d.%d' % (fw_latest[0], fw_latest[1]) +
-                            '. Suggested to update firmware, as you may experience errors')
+                            '. Suggested to update firmware, as you may experience errors' +
+                            '\nSee https://chipwhisperer.readthedocs.io/en/latest/api.html#firmware-update')
         return foundId
 
     def usbdev(self):
@@ -654,6 +700,14 @@ class NAEUSB(object):
 
         return self.usbseralizer.cmdWriteMem(addr, data)
 
+    def cmdWriteSam3U(self, addr, data):
+        """
+        Send command to write memory over external memory interface to FPGA. Automatically
+        decides to use control-transfer or bulk-endpoint transfer based on data length.
+        """
+
+        return self.usbseralizer.cmdWriteSam3U(addr, data)
+
     def writeBulkEP(self, data):
         """
         Write directoly to the bulk endpoint.
